/* xodsp - simple X triggering-oscilloscope-like program (port of vodsp)
 * Public domain by Russell Marks.
 *
 * This is a pretty crude Xlib port of vodsp - there's a lot of cookbook-ish
 * code here (pinched from zblast), so don't be surprised if it sucks. :-)
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>


Display *disp;
int screen,depth;
Window window;
GC gc;
Pixmap pixmap1,pixmap2,*pixmap;
unsigned long blackpix,whitepix;

/* XXX make resizable! */
int pwidth=320,pheight=129;

/* I was going to make this optional, but it looks terrible without it */
int doublebuf=1;



void x_init(int argc,char *argv[])
{
XWMHints wmhints;
XSizeHints hint;

disp=XOpenDisplay(NULL);	/* XXX should do getenv etc. */
if(disp==(Display *)NULL)
  {
  fprintf(stderr,"xodsp: couldn't open display\n");
  exit(1);
  }
screen=DefaultScreen(disp);
blackpix=BlackPixel(disp,screen);
whitepix=WhitePixel(disp,screen);

hint.min_width =hint.max_width =hint.base_width =pwidth;
hint.min_height=hint.max_height=hint.base_height=pheight;
hint.flags=PPosition|PSize|PMinSize|PMaxSize|PBaseSize;
depth=DefaultDepth(disp,screen);

window=XCreateSimpleWindow(disp,DefaultRootWindow(disp),
  0,0,pwidth,pheight,5,whitepix,blackpix);
pixmap1=XCreatePixmap(disp,window,pwidth,pheight,depth);
pixmap2=XCreatePixmap(disp,window,pwidth,pheight,depth);
if(doublebuf)
  pixmap=&pixmap1;
else
  pixmap=&window;

XSetStandardProperties(disp,window,"xodsp","xodsp",None,argv,argc,&hint);

gc=XCreateGC(disp,window,0,0);
XSetGraphicsExposures(disp,gc,0);
XSetBackground(disp,gc,blackpix);
XSetForeground(disp,gc,whitepix);

wmhints.flags=InputHint;
wmhints.input=True;
XSetWMHints(disp,window,&wmhints);
XSelectInput(disp,window,KeyPressMask | KeyReleaseMask | StructureNotifyMask |
                         VisibilityChangeMask | ExposureMask);
XMapRaised(disp,window);

if(doublebuf)
  {
  XSetForeground(disp,gc,blackpix);
  XFillRectangle(disp,pixmap1,gc,0,0,pwidth,pheight);
  XFillRectangle(disp,pixmap2,gc,0,0,pwidth,pheight);
  XSetForeground(disp,gc,whitepix);
  }

XSync(disp,0);
}


void x_uninit(void)
{
XFreeGC(disp,gc);
if(doublebuf)
  {
  XFreePixmap(disp,pixmap1);
  XFreePixmap(disp,pixmap2);
  }
XDestroyWindow(disp,window);
XCloseDisplay(disp);
}


void x_clearwin(void)
{
XSetForeground(disp,gc,blackpix);
XFillRectangle(disp,*pixmap,gc,0,0,pwidth,pheight);
}



int main(int argc,char *argv[])
{
static unsigned char buf[256];
int audio,tmp,x,y,oldy,f;
int start,maxdiff,maxpos;
int quit=0;

if((audio=open("/dev/dsp",O_RDONLY))<0)
  fprintf(stderr,"xodsp: couldn't open /dev/dsp.\n"),exit(1);

tmp=0x80008;		/* 8 frags, 256 bytes, 31.25/sec */
ioctl(audio,SNDCTL_DSP_SETFRAGMENT,&tmp);

x_init(argc,argv);

x_clearwin();

oldy=100;

while(!quit)
  {
  read(audio,buf,256);
  
  x_clearwin();

  /* trigger on the highest (then earliest) gradient between samples */
  start=0; maxdiff=maxpos=0;
  
  for(f=96;f>0;f--)
    {
    if(buf[f]>buf[f-1] && (tmp=(buf[f]-buf[f-1]))>=maxdiff)
      {
      maxdiff=tmp;
      maxpos=f-1;
      }
    }
  
  start=maxpos;

  if(start>0) oldy=pheight/2-((buf[start-1]-128)/2);
  
  XSetForeground(disp,gc,whitepix);
  for(x=2,f=start;f<256 && x<pwidth;f++,x+=2,oldy=y)
    {
    y=pheight/2-((buf[f]-128)/2);
    XDrawLine(disp,*pixmap,gc,x-2,oldy,x,y);
    }
  
  /* get all pending events */
  while(XPending(disp))
    {
    static XEvent event;
    static KeySym key;
    static char text[5];
    static int down,len;
    
    XNextEvent(disp,&event);
    switch(event.type)
      {
      case MappingNotify:
        XRefreshKeyboardMapping((XMappingEvent *)&event);
        break;
      case VisibilityNotify:
      case ConfigureNotify: /* when resized */
      case Expose:          /* when bit/all of window uncovered */
        /* do nothing, as we redisplay all the time. */
        break;
      case KeyPress:
      case KeyRelease:
        down=((event.type==KeyPress)?1:0);
        len=XLookupString((XKeyEvent *)&event,text,4,&key,0);
        switch(key)
          {
          case XK_Escape: case XK_q: case XK_x:
            if(down) quit=1;
            break;
          }
      }
    }
  
  /* actually draw the screen */
  if(doublebuf)
    {
    XCopyArea(disp,*pixmap,window,gc,0,0,pwidth,pheight,0,0);
    if(pixmap==&pixmap1) pixmap=&pixmap2; else pixmap=&pixmap1;
    }
  
  XSync(disp,0);
  
  if(doublebuf)
    x_clearwin();
  }

x_uninit();
exit(0);
}
